#!/usr/bin/env python
''' riskiq.cli.blacklist
Blacklist endpoint CLI scripts
'''
from argparse import ArgumentParser

from riskiq.api import Client

FILTERS = ('blackhole', 'sakura', 'exploitKit')
CONFIDENCES = ('H', 'M', 'L')

from riskiq.cli.blacklist import (
    lookup, incident, incidentlist, bl_list, malware
)

MAIN_FUNC = {
    'lookup': lookup.run,
    'incident': incident.run,
    'incidentlist': incidentlist.run,
    'list': bl_list.run,
    'malware': malware.run,
}

def main():
    parser = ArgumentParser()
    parser.add_argument('--dump-requests', action='store_true')
    parser.add_argument('--stix',
        help='output results to STIX file (requires riskiq[stix] package)')
    subs = parser.add_subparsers(dest='cmd')
    for module in (lookup, incident, incidentlist, bl_list, malware):
        module.add_parser(subs)
    args = parser.parse_args()

    client = Client.from_config()
    if args.dump_requests:
        client._dump_requests()

    kwargs = {}
    for kwarg in ('as_json', 'oneline', 'stix', 'days', 'start', 'end',
            'verbose', 'timeout', 'six_hours', 'filter', 'confidence',
            'template'):
        if hasattr(args, kwarg):
            kwargs[kwarg] = getattr(args, kwarg)
    
    sub_main = MAIN_FUNC[args.cmd]
    sub_main(client, args, kwargs)

if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        pass



#!/usr/bin/env python
import os
import sys
import json

from riskiq.api import Client
from riskiq.render import renderer

def main():
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument('--domain', '-d')
    parser.add_argument('--email', '-e')
    parser.add_argument('--name-server', '-n')
    parser.add_argument('--max-results', '-m', type=int, default=100)
    parser.add_argument('-j', '--json', action="store_true", dest='as_json',
        help="Output as JSON")
    args = parser.parse_args()
    client = Client.from_config()
    results = client.post_whois(domain=args.domain, email=args.email, 
        name_server=args.name_server, max_results=args.max_results)
    if args.as_json:
        print(json.dumps(results, indent=4))
    else:
        print(renderer(results, 'whois/whois'))

if __name__ == '__main__':
    main()



#!/usr/bin/env python

import sys
import json

from riskiq.api import Client
from riskiq.render import renderer
from riskiq.cli import util


def lp_get(client, md5_hash, whois=None, as_json=False):
    data = client.get_landing_page(md5_hash, whois=whois)
    if as_json:
        print(json.dumps(data, indent=4))
    elif data:
        print(renderer({'landingPage': [data]}, 'landingpage/crawled'))

def lp_submit(client, url, as_json=False, **kwargs):
    data = client.submit_landing_page(url, **kwargs)
    if as_json:
        print(json.dumps(data, indent=4))
    elif data:
        print(renderer({'landingPage': [data]}, 'landingpage/crawled'))

def lp_crawled(client, as_json=None, **kwargs):
    data = client.get_landing_page_crawled(**kwargs)
    if as_json:
        print(json.dumps(data, indent=4))
    elif data:
        print(renderer(data, 'landingpage/crawled'))

def lp_flagged(client, as_json=None, **kwargs):
    data = client.get_landing_page_flagged(**kwargs)
    if as_json:
        print(json.dumps(data, indent=4))
    elif data:
        print(renderer(data, 'landingpage/crawled'))

def lp_submit_bulk(client, urls, as_json=False, **kwargs):
    entries = []
    for url in urls:
        entries += [{'url': url}]
    for entry in entries:
        entry.update(kwargs)
    data = client.submit_landing_page_bulk(entries)
    if as_json:
        print(json.dumps(data, indent=4))
    elif data:
        print(renderer(data, 'landingpage/bulk'))

def lp_binary(client, as_json=False, **kwargs):
    data = client.get_landing_page_malicious_binary(**kwargs)
    if as_json:
        print(json.dumps(data, indent=4))
    elif data:
        print(renderer(data, 'landingpage/crawled'))

def lp_projects(client, as_json=False):
    data = client.get_landing_page_projects()
    if as_json:
        print(json.dumps(data, indent=4))
    elif data:
        print(renderer(data, 'landingpage/projects'))

def main():
    import argparse
    parser = argparse.ArgumentParser()
    subs = parser.add_subparsers(dest='cmd')

    get_parser = subs.add_parser('get',
        help='Retrieve a single landingpage by MD5 hash')
    get_parser.add_argument('md5_hashes', nargs='+')
    get_parser.add_argument('--whois', '-w', action='store_true',
        help='whether to include whois information')
    get_parser.add_argument('-j', '--json', action="store_true", dest='as_json',
        help="Output as JSON")

    submit_parser = subs.add_parser('submit',
        help='Submit at least one or many landing pages.')
    submit_parser.add_argument('urls', nargs='+')
    submit_parser.add_argument('--project', '-p',
        help='Project name to submit to')
    submit_parser.add_argument('--keyword', '-k',
        help='Optional Keyword')
    submit_parser.add_argument('--md5', '-m',
        help='Optional MD5 representing the canonical ID')
    submit_parser.add_argument('--pingback-url', '-P',
        help='Optional URL to be GET requested upon completion of analysis')
    submit_parser.add_argument('--fields', '-f', nargs='*',
        help='Optional list of custom fields eg -f foo=bar alpha=beta')
    submit_parser.add_argument('-j', '--json', action="store_true", dest='as_json',
        help="Output as JSON")
    
    crawled_parser = subs.add_parser('crawled',
        help='List landing pages by crawl date - maximum of 100')
    crawled_parser.add_argument('--whois', '-w', action='store_true',
        help='whether to include whois information')
    crawled_parser.add_argument('--days', '-d', default=None, type=int,
        help='days to query')
    crawled_parser.add_argument('--start', '-s', default=None,
        help='start datetime in yyyy-mm-dd HH:MM:SS format, or "today HH:MM:SS"')
    crawled_parser.add_argument('--end', '-e', default=None,
        help='end datetime in yyyy-mm-dd HH:MM:SS format, or "today HH:MM:SS"')
    crawled_parser.add_argument('-j', '--json', action="store_true",
        dest='as_json', help="Output as JSON")

    flagged_parser = subs.add_parser('flagged',
        help='List landing pages by known profile creation date - '
            'maximum of 100')
    flagged_parser.add_argument('--whois', '-w', action='store_true',
        help='whether to include whois information')
    flagged_parser.add_argument('--days', '-d', default=None, type=int,
        help='days to query')
    flagged_parser.add_argument('--start', '-s', default=None,
        help='start datetime in yyyy-mm-dd HH:MM:SS format, or "today HH:MM:SS"')
    flagged_parser.add_argument('--end', '-e', default=None,
        help='end datetime in yyyy-mm-dd HH:MM:SS format, or "today HH:MM:SS"')
    flagged_parser.add_argument('-j', '--json', action="store_true",
        dest='as_json', help="Output as JSON")

    binary_parser = subs.add_parser('binary',
        help='List landing pages with malicious binary incidents. '
            'A malicious binary is any non-text file that is suspected of '
            'containing malware or exploit code. A landing page is linked to '
            'any such binary that is embedded or easily reachable from it.'
    )
    binary_parser.add_argument('--whois', '-w', action='store_true',
        help='whether to include whois information')
    binary_parser.add_argument('--days', '-d', default=1, type=int,
        help='days to query')
    binary_parser.add_argument('--start', '-s', default=None,
        help='start datetime in yyyy-mm-dd HH:MM:SS format, or "today HH:MM:SS"')
    binary_parser.add_argument('--end', '-e', default=None,
        help='end datetime in yyyy-mm-dd HH:MM:SS format, or "today HH:MM:SS"')
    binary_parser.add_argument('-j', '--json', action="store_true",
        dest='as_json', help="Output as JSON")

    pjs_parser = subs.add_parser('projects',
        help='List all projects that landing pages may be submitted to.')
    pjs_parser.add_argument('-j', '--json', action="store_true", dest='as_json',
        help="Output as JSON")

    args = parser.parse_args()
    client = Client.from_config()

    kwargs = {'as_json': args.as_json}

    if hasattr(args, 'whois'):
        kwargs['whois'] = args.whois
    if hasattr(args, 'days'):
        kwargs['days'] = args.days
        kwargs['start'] = args.start
        kwargs['end'] = args.end
    if args.cmd == 'get':
        md5_hashes = util.stdin(args.md5_hashes)
        for md5_hash in md5_hashes:
            lp_get(client, md5_hash, **kwargs)
    elif args.cmd == 'submit':
        urls = util.stdin(args.urls)
        kwargs.update({
            'keyword': args.keyword,
            'md5_hash': args.md5,
            'pingback_url': args.pingback_url,
            'project_name': args.project,
        })
        if args.fields:
            kwargs.update({'fields': dict([f.split('=') for f in args.fields])})
        if len(urls) == 1:
            lp_submit(client, urls[0], **kwargs)
        else:
            lp_submit_bulk(client, urls, **kwargs)
    elif args.cmd == 'crawled':
        lp_crawled(client, **kwargs)
    elif args.cmd == 'flagged':
        lp_flagged(client, **kwargs)
    elif args.cmd == 'binary':
        lp_binary(client, **kwargs)
    elif args.cmd == 'projects':
        lp_projects(client, **kwargs)

if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        pass



#!/usr/bin/env python
import sys
import json
from datetime import datetime, timedelta
from riskiq.render import renderer

try:
    from riskiq import blacklist_stix
    NO_STIX = False
except ImportError:
    NO_STIX = True

def stdin(argv):
    parse_stdin = ('-' in argv)
    if parse_stdin:
        argv = [x for x in argv if x != '-']
        input_lines = sys.stdin.readlines()
        argv += [line.strip() for line in input_lines if line.strip()]
    return argv

def six_hours():
    return (datetime.now() - timedelta(hours=6)).strftime('%Y-%m-%d %H:%M:%S')

def dump_data(data, temp, kwargs):
    # Dump to --stix path
    if kwargs.get('stix'):
        dump_stix(data, kwargs['stix'])
    elif kwargs.get('as_json'):
        print(json.dumps(data, indent=4))
    elif data:
        print(
            renderer(data, temp, 
                oneline=kwargs.get('oneline', False),
                verbose=kwargs.get('verbose', False),
                custom_template=kwargs.get('template'),
            )
        )

def templated(temp, yielding=False):
    def deco(func):
        # Simple return of one set of data
        def wrapped(*args, **kwargs):
            data, kwargs2 = func(*args, **kwargs)
            dump_data(data, temp, kwargs2)
        # Handles case where it yields multiple data points
        def wrapped_yielding(*args, **kwargs):
            all_data = {}
            kwargs2 = {}
            for data, kwargs2 in func(*args, **kwargs):
                all_data.update(data)
            dump_data(all_data, temp, kwargs2)
        if yielding:
            return wrapped_yielding
        return wrapped
    return deco
         
def dump_stix(data, path):
    if NO_STIX:
        raise RuntimeError('Please install riskiq[stix]')
    data = blacklist_stix.load_bldata(data)
    output_xml = blacklist_stix.stix_xml(data)
    blacklist_stix.dump_xml(path, output_xml)
    print('Dumped STIX XML to {}'.format(path))




#!/usr/bin/env python
''' riskiq.cli.blacklist.bl_list
'''
from riskiq.cli.util import templated, six_hours
from riskiq.cli.parser import add_timerange_args, add_render_args
from riskiq.cli.blacklist import FILTERS

def add_parser(subs):
    parser = subs.add_parser('list', help = 'query blacklisted resources')
    parser.add_argument('--filter', '-f', default=None,
        help='filter to one of "blackhole", "sakura" or "exploitKit"')
    add_timerange_args(parser)
    add_render_args(parser)

def main(client, args, kwargs):
    data = client.get_blacklist_incident_list(**kwargs)
    return data, kwargs

@templated('blacklist/malware')
def run(client, args, kwargs):
    if kwargs.get('six_hours'):
        kwargs['start'] = six_hours()
        del kwargs['six_hours']
    blacklist_filter = kwargs['filter']
    del kwargs['filter']
    if blacklist_filter not in (None, ) + FILTERS:
        raise ValueError('Invalid filter. Must be one of %s' % str(FILTERS))
    data = client.get_blacklist_list(blacklist_filter=blacklist_filter,
        **kwargs)
    return data, kwargs



#!/usr/bin/env python
import json
import argparse
from datetime import datetime

from stix.core import STIXPackage, STIXHeader
from stix.utils import set_id_namespace
from stix.common.vocabs import PackageIntent
from stix.common import InformationSource
from cybox.common import Time
from stix.indicator import Indicator
from cybox.objects.uri_object import URI

def load_bldata(path):
    # Read the json data
    if isinstance(path, dict):
        data = path
    else:
        with open(path) as blFile:
            data = json.load(blFile)
    if 'resources' in data:
        return data['resources']
    elif 'incident' in data:
        return [x['resource'] for x in data['incident']]
    else:
        raise RuntimeError('Invalid JSON file. Please use riq-blacklist '
            'malware output, or riq-blacklist incidents')

def dump_xml(out_path, output_xml):
    if out_path == '-':
        print(output_xml)
    else:
        with open(out_path, 'w') as f:
            f.write(output_xml)

def stix_xml(bldata):
    # Create the STIX Package and Header objects
    stix_package = STIXPackage()
    stix_header = STIXHeader()
    # Set the description
    stix_header.description = "RiskIQ Blacklist Data - STIX Format"
    # Set the namespace
    NAMESPACE = {"http://www.riskiq.com" : "RiskIQ"}
    set_id_namespace(NAMESPACE) 
    # Set the produced time to now
    stix_header.information_source = InformationSource()
    stix_header.information_source.time = Time()
    stix_header.information_source.time.produced_time = datetime.now()
    # Create the STIX Package
    stix_package = STIXPackage()
    # Build document
    stix_package.stix_header = stix_header
    # Build the Package Intent
    stix_header.package_intents.append(PackageIntent.TERM_INDICATORS)

    # Build the indicator
    indicator = Indicator()
    indicator.title = "List of Malicious URLs detected by RiskIQ - Malware, Phishing, and Spam"
    indicator.add_indicator_type("URL Watchlist")
    for datum in bldata:
        url = URI()
        url.value = ""
        url.value = datum['url']
        url.type_ =  URI.TYPE_URL
        url.condition = "Equals"
        indicator.add_observable(url)

    stix_package.add_indicator(indicator)
    return stix_package.to_xml()

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('json_file', help='path to blacklist json file')
    parser.add_argument('--output', '-o', help='path to output, default:stdout',
        default='-')
    args = parser.parse_args()
    bldata = load_bldata(args.json_file)
    output_xml = stix_xml(bldata)
    dump_xml(args.output, output_xml)

if __name__ == '__main__':
    main()



import os

from jinja2 import Template, Environment, PackageLoader

def renderer(data, template_file, custom_template=None,
        verbose=False, oneline=False):
    """
    Render the template with supplied context.
    Example template_file: "blacklist/lookup"

    """
    if custom_template:
        with open(custom_template) as f:
            template = Template(f.read())
        return template.render(data=data, verbose=verbose).encode('utf-8').strip()
    if oneline:
        template_file += '_oneline'
    env = Environment(loader=PackageLoader('riskiq', 'templates'))
    template = env.get_template(template_file)
    return template.render(data=data, verbose=verbose).encode('utf-8').strip()



#!/usr/bin/env python
"""
RiskIQ API
"""
__author__ = 'RiskIQ Research'
__version__ = '0.4.9'

import json
import sys
from datetime import timedelta, datetime

# sys.version_info.major exists in python>=2.7, but just a tuple python<2.7 :(
PY2 = (sys.version_info[0] == 2)
PY3 = (sys.version_info[0] == 3)
if PY2:
    import httplib
    str_type = eval('basestring')
else:
    str_type = str

import requests

from riskiq.config import Config

# Acceptable string time format for all requests
TIME_FORMAT = '%Y-%m-%d %H:%M:%S'
TIME_FORMAT_DAY = '%Y-%m-%d 00:00:00'
TIME_FORMAT_ISO = '%Y-%m-%dT%H:%M:%S.000-0000'

INVENTORY_ASSET_TYPES = ['ALL', 'WEB_SITE', 'NAME_SERVER', 'MAIL_SERVER',
                         'HOST', 'DOMAIN', 'IP_BLOCK', 'ASN', 'SSL_CERT',
                         'CONTACT']

def today():
    """
    Generates a date string for today.

    :return: Date string of today in "yyyy-mm-dd" format, accepted by API
    """
    return datetime.strftime(datetime.now(), '%Y-%m-%d')


def format_date(dt, day=False):
    """
    Generates a date string in the required format from a datetime object.

    :param dt: Datetime object
    :param day: Bool, whether to take the floor of the day
        (1 means beginning of today since midnight)
    :return: string in acceptable date format
    """
    fmt = TIME_FORMAT
    if day:
        fmt = TIME_FORMAT_DAY
    return datetime.strftime(dt, fmt)


def date_range(days=1, start=None, end=None):
    """
    Generate a start date and an end date based off of how many days.

    :param days: How many days to include from today(for generating 30 day time windows, etc.)
    :param start: Override start date.
    :param end: Override end date
    :return: (start, end) tuple of strings in acceptable date format
    """
    if not any([days, start, end]):
        return None, None
    if start is None:
        start = format_date(datetime.now() - timedelta(days=days-1), day=True)
    elif isinstance(start, datetime):
        start = format_date(start)
    elif isinstance(start, str_type):
        start = start.replace('today', today())
    if end is None:
        end = format_date(datetime.now())
    elif isinstance(end, datetime):
        end = format_date(end)
    elif isinstance(end, str_type):
        end = end.replace('today', today())
    return start, end


def set_if(dictionary, key, value):
    if value is not None:
        dictionary[key] = value


class FilterField:
    ''' Used for filter fields in /inventory/search params '''
    AssetType = 'assetType'


class FilterValue:
    ''' Used for filter values in /inventory/search params '''
    WebSite = 'WEB_SITE'


class FilterOperation:
    ''' Used for filter operations in /inventory/search params '''
    Equals = 'EQ'
    NotEqual = 'NE'
    Like = 'LIKE'
    NotLike = 'NOT_LIKE'
    Contains = 'CONTAINS'
    NotContains = 'NOT_CONTAINS'
    Null = 'NULL'
    NotNull = 'NOT_NULL'
    Between = 'BETWEEN'
    In = 'IN'
    NotIn = 'NOT_IN'
    GreaterThan = 'GT'
    GreaterThanOrEqual = 'GTE'
    LessThan = 'LT'
    LessThanOrEqual = 'LTE'


class Client(object):
    """
    RiskIQ API Client

    Example:
    ::

        from riskiq.api import Client
        # Put credentials here.
        token, key = None, None
        client = Client(token, key)

        # Submit URLs to your project
        urls = ['http://evilexample.com/evil.php?shell=true', ...]
        client.submit_landing_page_bulk(urls, project_name='Example')

        # Get blacklist list from varying date ranges
        client.get_blacklist_list(days=5)
        data = client.get_blacklist_list(
            start="2014/08/01 00:00:00", end="today 00:00:00"
        )
        results = data['resources']
        all_malware = [x for x in results if x['malware']]
    """

    # Default seconds until socket timeout on all GETs
    # override with self._get(..., timeout=10)
    TIMEOUT = 60

    def __init__(self, token, key, server='ws.riskiq.net', version='v1',
                 http_proxy=None, https_proxy=None):
        self.api_base = 'https://%s/%s' % (server, version)
        self.auth = (token, key)
        self.headers = {
            'Accept': 'application/json',
            'Content-Type': 'application/json',
        }
        self.proxies = {}
        if http_proxy:
            self.proxies['http'] = http_proxy
        if https_proxy:
            self.proxies['https'] = https_proxy

    @classmethod
    def from_config(cls):
        config = Config()
        client = cls(
            token=config.get('api_token'),
            key=config.get('api_private_key'),
            server=config.get('api_server'),
            version=config.get('api_version'),
            http_proxy=config.get('http_proxy'),
            https_proxy=config.get('https_proxy'),
        )
        return client

    def _endpoint(self, endpoint, action, *url_args):
        """
        Return the URL for the action

        :param endpoint: The controller
        :param action: The action provided by the controller
        :param url_args: Additional endpoints(for endpoints that take part of the url as option)
        :return: Full URL for the requested action
        """
        api_url = "/".join((self.api_base, endpoint, action))
        if url_args:
            api_url += "/".join(url_args)
        return api_url

    def _dump_requests(self):
        ''' Disable for Python3 '''
        if PY2:
            self._old_send = httplib.HTTPConnection.send
            old_send = self._old_send

            def new_send(self, data):
                print(data)
                return old_send(self, data)
            httplib.HTTPConnection.send = new_send

    def _undump_requests(self):
        ''' Disable for Python3 '''
        if PY2:
            httplib.HTTPConnection.send = self._old_send

    def _json(self, response):
        """
        JSON response from server

        :param response: Response from the server
        :throws ValueError: from requests' response.json() error
        :return: response deserialized from JSON
        """
        if response.status_code == 204:
            return None
        try:
            return response.json()
        except ValueError as e:
            raise ValueError(
                'Exception: %s\n'
                'request: %s, response code: %s, response: %s' % (
                    str(e), response.request.url, response.status_code,
                    response.content,
                )
            )

    def _get(self, endpoint, action, *url_args, **url_params):
        """
        Request API Endpoint - for GET methods.

        :param endpoint: Endpoint
        :param action: Endpoint Action
        :param url_args: Additional endpoints(for endpoints that take part of the url as option)
        :param url_params: Parameters to pass to url, typically query string
        :return: response deserialized from JSON
        """
        api_url = self._endpoint(endpoint, action, *url_args)
        if 'timeout' in url_params:
            timeout = url_params['timeout']
            del url_params['timeout']
        else:
            timeout = Client.TIMEOUT
        kwargs = {'auth': self.auth, 'headers': self.headers, 'params': url_params,
                  'timeout': timeout, 'verify': True}
        if self.proxies:
            kwargs['proxies'] = self.proxies
        response = requests.get(api_url, **kwargs)
        return self._json(response)

    def _post(self, endpoint, action, data, *url_args, **url_params):
        """
        Submit to API Endpoint - for POST methods.

        :param endpoint: Endpoint
        :param action: Endpoint Action
        :param url_args: Additional endpoints(for endpoints that take part of the url as option)
        :param url_params: Parameters to pass to url, typically query string
        :return: response deserialized from JSON
        """
        api_url = self._endpoint(endpoint, action, *url_args)
        data = json.dumps(data)
        kwargs = {'auth': self.auth, 'headers': self.headers, 'params': url_params,
                  'verify': True, 'data': data}
        if self.proxies:
            kwargs['proxies'] = self.proxies
        response = requests.post(api_url, **kwargs)
        return self._json(response)

    def get_affiliate_campaign_summary(self, days=1, start=None, end=None):
        """
        Return the affiliate campaign summary report for the given date range.

        :param days: How many days to include from today(for generating 30 day time windows, etc.)
        :param start: Override start date.
        :param end: Override end date
        :return: data containing the number of results and the objects
        """
        start, end = date_range(days, start, end)
        return self._get('affiliate', 'campaignSummary',
                         startDateInclusive=start, endDateExclusive=end)

    def _landing_page_entry(self, url=None, keyword=None,
                            md5_hash=None, project_name=None, pingback_url=None,
                            fields=None):
        """
        Build the dictionary for a single landing_page submission.

        :param url: Url to submit. Only required parameter.
        :param keyword: Optional Keyword for this landing page.
        :param md5_hash: Optional MD5 representing the canonical ID for this
            landing page
        :param project_name: Optional Project name to submit landing page to
        :param pingback_url: Optional URL to be GET requested upon completion
            of analysis of the landing page
        :param fields: Optional dictionary of custom fields
        :return: returns dictionary usable for requests
        """
        if url is None:
            raise ValueError('url param is required in landing_page submission')
        data = {}
        if url:
            data['url'] = url
        if keyword:
            data['keyword'] = keyword
        if md5_hash:
            data['md5'] = md5_hash
        if project_name:
            data['projectName'] = project_name
        if pingback_url:
            data['pingbackUrl'] = pingback_url
        if fields:
            data['fields'] = [
                {'name': t[0], 'value': t[1]}
                for t in fields.items()
            ]
        return data

    def get_affiliate_incident_list(self, known_profile=None, max_results=None,
                                    days=1, start=None, end=None):
        """
        Return the affiliate campaign summary report for the given date range.

        :param known_profile: Bool, only return incidents that match a known profile
        :param max_results: maximum number of results to return
        :param days: How many days to include from today(for generating 30 day time windows, etc.)
        :param start: Override start date.
        :param end: Override end date
        :return: data containing the number of results and the objects
        """
        start, end = date_range(days, start, end)
        kwargs = {
            'startDateInclusive': start,
            'endDateExclusive': end,
        }
        if known_profile is not None:
            kwargs['knownProfile'] = known_profile
        if max_results is not None:
            kwargs['maxResults'] = max_results
        return self._get('affiliate/incident', 'list', **kwargs)

    def get_binary_list(self, virus_total_only=None, client_workspace_only=None,
                        days=1, start=None, end=None):
        """
        Return a list of all binaries in date range

        :param virus_total_only: Bool, only include those flagged by VT
        :param client_workspace_only: Bool, only include those found in crawls
        :param days: How many days to include from today(for generating 30 day time windows, etc.)
        :param start: Override start date.
        :param end: Override end date
        :return: data containing the number of results and the objects
        """
        start, end = date_range(days, start, end)
        kwargs = {
            'startDateInclusive': start,
            'endDateExclusive': end,
        }
        if virus_total_only is not None:
            kwargs['virus_total_only'] = virus_total_only
        if client_workspace_only is not None:
            kwargs['client_workspace_only'] = client_workspace_only
        return self._get('binary', 'list', **kwargs)

    def get_binary_data(self, md5_hash):
        """
        Query for the binary encoded base64 with the given md5 hash

        :param md5_hash: md5 hash of the binary data
        :return: object containing a dict with 'data' key encoded in base64
        """
        return self._get('binary', 'data', md5=md5_hash)

    def get_blacklist_lookup(self, url):
        """
        Query blacklist on url.

        :param url: URL to query blacklist on.
        :return: Blacklist Dict
        """
        result = self._get('blacklist', 'lookup', url=url)
        if result and 'description' not in result:
            result['description'] = ''
        return result

    def get_blacklist_lookup_bulk(self, urls):
        """
        Query blacklist urls in bulk.
        At least one url must be specified.

        :param urls: Array of URLs to query blacklist on.
        :return: Array of Blacklist Dicts
        """
        result = self._get('blacklist', 'bulkLookup', urls=",".join(urls))
        if result and 'lookup' not in result:
            result['lookup'] = []
        return result['lookup']

    def get_blacklist_incident(self, url, start_index=None, max_results=None,
                               **kwargs):
        """
        Query blacklist incidents by url.

        :param url: URL to query blacklist on.
        :return: Blacklist incident
        """
        url_params = {'url': url}
        if start_index is not None:
            url_params['startIndex'] = start_index
        if max_results is not None:
            url_params['maxResults'] = max_results
        return self._get('blacklist', 'incident', **url_params)

    def get_blacklist_incident_list(self, all_workspace_crawls=None, days=1,
                                    start=None, end=None, timeout=None,
                                    **kwargs):
        """
        Query blacklist incidents

        :param url: list of blacklist incidents within timeframe
        :param all_workspace_crawls:False by default, filtered to crawls that
            are either landing pages, site scanning, or matching a brand
            classifier
        :param days: How many days to include from today(for generating 30 day time windows, etc.)
        :param start: Override start date.
        :param end: Override end date
        :return: Blacklist list
        """
        start, end = date_range(days, start, end)
        url_params = {
            'startDateInclusive': start,
            'endDateExclusive': end,
        }
        if all_workspace_crawls is not None:
            url_params['all_workspace_crawls'] = all_workspace_crawls
        if timeout is not None:
            url_params['timeout'] = timeout
        return self._get('blacklist/incident', 'list', **url_params)

    def get_blacklist_list(self, blacklist_filter=None, days=1, start=None,
                           end=None, **kwargs):
        """
        Query blacklisted resources

        :param blacklist_filter: None, or one of
            'blackhole', 'sakura', 'exploitKit'
        :param days: How many days to include from today(for generating 30 day time windows, etc.)
        :param start: Override start date.
        :param end: Override end date
        :return: all blacklisted resources
        """
        start, end = date_range(days, start, end)
        url_params = {
            'startDateInclusive': start,
            'endDateExclusive': end,
        }
        if blacklist_filter is not None:
            url_params['filter'] = blacklist_filter
        return self._get('blacklist', 'list', **url_params)

    def get_blacklist_malware(self, blacklist_filter=None, confidence=None,
                              days=1, start=None, end=None, **kwargs):
        """
        Query for all discovered malware resources generated within a
        particular period.

        :param blacklist_filter: None, or one of
            'blackhole', 'sakura', 'exploitKit'
        :param confidence: to restrict the result set by malicious probability
            'H', 'M', 'L' (high, medium, low)
        :param days: How many days to include from today(for generating 30 day
            time windows, etc.)
        :param start: Override start date.
        :param end: Override end date
        :return: all blacklisted resources
        """
        start, end = date_range(days, start, end)
        url_params = {
            'startDateInclusive': start,
            'endDateExclusive': end,
        }
        if blacklist_filter is not None:
            url_params['filter'] = blacklist_filter
        if confidence is not None:
            url_params['confidence'] = confidence
        return self._get('blacklist', 'malware', **url_params)

    def __get_blacklist_exploit_binary(self, days=1, start=None, end=None,
                                       **kwargs):
        """
        Query for all PE format binaries on webpages used for exploitation

        :param days: How many days to include from today(for generating 30 day time windows, etc.)
        :param start: Override start date.
        :param end: Override end date
        :return: all binaries
        """
        raise NotImplementedError('Not implemented server-side')
        start, end = date_range(days, start, end)
        url_params = {
            'startDateInclusive': start,
            'endDateExclusive': end,
        }
        return self._get('blacklist', 'exploitBinary', **url_params)

    def get_crawl_volume_daily_summary(self, days=1, start=None, end=None):
        """
        Query for the crawl volume daily summary report for the date range

        :param days: How many days to include from today(for generating 30 day time windows, etc.)
        :param start: Override start date.
        :param end: Override end date
        :return: crawl volume daily summary
        """
        start, end = date_range(days, start, end)
        kwargs = {
            'startDateInclusive': start,
            'endDateInclusive': end,
        }
        return self._get('crawlVolume', 'dailySummary', **kwargs)

    def get_dns_data_by_name(self, name, rrtype=None, maxresults=1000):
        """
        Get the passive dns results by hostname.

        :param name: hostname to query. Can also use wildcards, e.g. *.test.com
        :param rrtype: Record Type to limit searches to
        :param maxresults: Max Results to Return(default 1,000)
        :return: return a JSON object of the data
        """
        return self._get('dns', 'name', name=name, rrType=rrtype,
                         maxResults=maxresults)

    def get_dns_data_by_ip(self, ip, rrtype=None, maxresults=1000):
        """
        Get the passive dns results by ip address, query data

        :param ip: IP address of query, can also include wildcard, e.g. 192.168.0.*
        :param rrtype: Record Type to limit searches to
        :param maxresults: Max Results to Return(default 1,000)
        :return: return a JSON object of the data
        """
        return self._get('dns', 'data', ip=ip, rrType=rrtype,
                         maxResults=maxresults)

    def get_dns_ptr_by_ip(self, ip, rrtype=None, maxresults=1000):
        """
        Get the reverse dns of a particular IP.

        :param ip: IP Address to Query.
        :param rrtype: Record Type to limit searches to
        :param maxresults: Max Results to Return(default 1,000)
        :return: return a JSON object of the data
        """
        return self._get('dns', 'name', ip=ip, rrType=rrtype,
                         maxResults=maxresults)

    def get_dns_data_by_data(self, hostname, rrtype=None, maxresults=1000):
        """
        Get the passive dns results by ip address, query data

        :param ip: IP address of query, can also include wildcard, e.g. 192.168.0.*
        :param rrtype: Record Type to limit searches to
        :param maxresults: Max Results to Return(default 1,000)
        :return: return a JSON object of the data
        """
        return self._get('dns', 'data', name=hostname, rrType=rrtype,
                         maxResults=maxresults)

    def get_landing_page(self, md5_hash, whois=None):
        """
        Retrieve a single landing page by MD5.

        :param md5_hash: md5 of the landing page
        :param whois: Bool, whether to include whois information
        :return: landing page data
        """
        kwargs = {}
        if whois is not None:
            kwargs['whois'] = whois
        return self._get('landingPage', md5_hash, **kwargs)

    def submit_landing_page(self, url, **kwargs):
        """
        Submit a single landing page.

        :param url: Url to submit. Only required parameter.
        :param keyword: Optional Keyword for this landing page.
        :param md5_hash: Optional MD5 representing the canonical ID for this
            landing page
        :param project_name: Optional Project name to submit landing page to
        :param pingback_url: Optional URL to be GET requested upon completion
            of analysis of the landing page
        :param fields: Optional dictionary of custom fields
        :return: returns json of landing page.
        """
        kwargs.update({'url': url})
        data = self._landing_page_entry(**kwargs)
        return self._post('landingPage', '', data)

    def get_landing_page_crawled(self, whois=None, days=None, start=None,
                                 end=None):
        """
        List landing pages by crawl date - maximum of 100

        :param whois: Bool, whether to include whois information
        :param days: How many days you want to grab
        :param start: Which date to start from, use time_format.
        :param end: Date to end, use time_format.
        :return: landing page data
        """
        start, end = date_range(days, start, end)
        if any([days, start, end]):
            kwargs = {'start': start, 'end': end}
        else:
            kwargs = {}
        if whois is not None:
            kwargs['whois'] = whois
        return self._get('landingPage', 'crawled', **kwargs)

    def get_landing_page_flagged(self, whois=None, days=None, start=None,
                                 end=None):
        """
        List landing pages by known profile creation date - maximum of 100

        :param whois: Bool, whether to include whois information
        :param days: How many days you want to grab
        :param start: Which date to start from, use time_format.
        :param end: Date to end, use time_format.
        :return: landing page data
        """
        start, end = date_range(days, start, end)
        if any([days, start, end]):
            kwargs = {'start': start, 'end': end}
        else:
            kwargs = {}
        if whois is not None:
            kwargs['whois'] = whois
        return self._get('landingPage', 'flagged', **kwargs)

    def __submit_landing_page_urls(self, urls, **kwargs):
        """
        Submit landing pages in bulk. This is the old form of
        submit_landing_page_bulk, and does not offer the full features of the
        API. See submit_landing_page_bulk.

        :param urls: Urls to submit.
        :param project_name: Project name to submit landing page to
        :return: returns json of landing page.
        """
        project_name = kwargs.get('project_name')
        if project_name is not None:
            data = {
                'entry': [
                    {
                        'url': url,
                        'projectName': project_name
                    }
                    for url in urls
                ]
            }
        else:
            data = {'entry': [{'url': url} for url in urls]}
        return self._post('landingPage', 'bulk', data)

    def submit_landing_page_bulk(self, entries, **kwargs):
        """
        Submit landing pages in bulk
        At least url must be specified.

        :param entries: list of dictionaries specifying the below

        :entry_key url: Url to submit.
        :entry_key keyword: Optional Keyword for this landing page.
        :entry_key md5_hash: Optional MD5 representing the canonical ID for this
            landing page
        :entry_key project_name: Optional Project name to submit landing page to
        :entry_key pingback_url: Optional URL to be GET requested upon completion
            of analysis of the landing page
        :entry_key fields: Optional dictionary of custom fields
        :return: returns json of landing page bulk request.
        """
        # Check to see if entries is a list of urls via the old API call.
        # We need to check this for backwards compatibility.
        if len(entries) > 0 and isinstance(entries[0], str_type):
            return self.__submit_landing_page_urls(entries, **kwargs)
        # It's new style, so build it from the list of dictionaries.
        data = {'entry': [
            self._landing_page_entry(**entry)
            for entry in entries
        ]}
        return self._post('landingPage', 'bulk', data)

    def get_landing_page_malicious_binary(self, whois=None, days=1, start=None,
                                          end=None):
        """
        List landing pages with malicious binary incidents.

        :param whois: Bool, whether to include whois information
        :param days: How many days you want to grab
        :param start: Which date to start from, use time_format.
        :param end: Date to end, use time_format.
        :return: landing page data
        """
        start, end = date_range(days, start, end)
        url_params = {'start': start, 'end': end}
        if whois is not None:
            url_params['whois'] = whois
        return self._get('landingPage', 'maliciousBinary', **url_params)

    def get_landing_page_projects(self):
        """
        List all projects that landing pages may be submitted to.

        :return: all projects
        """
        return self._get('landingPage', 'projects')

    def get_android(self, package_name):
        """
        Retrieve an android application by package name.
        If the app is not found, 404 NOT FOUND is returned.

        :param package_name: name of android package
        :return: the requested app
        """
        return self._get('mobile/android', package_name)

    def get_android_lookup(self, url):
        """
        Retrieve an android app by store URL.
        The store URL should be of the form
        https://play.google.com/store/apps/details?id=[package name]

        :param url: The store URL
        :return: app details
        """
        return self._get('mobile/android', 'lookup', url=url)

    def get_mobile_incident(self, incident_id):
        """
        Retrieve an mobile app incident by ID.
        If the incident is not found, 404 NOT FOUND is returned.

        :param incident_id: Long int ID
        :return: mobile incident
        """
        return self._get('mobile/incident', '%d' % incident_id)

    def get_mobile_incident_list(self, days=1, start=None, end=None):
        """
        List app incidents by their incident creation date.

        :param days: How many days you want to grab
        :param start: Which date to start from, use time_format.
        :param end: Date to end, use time_format.
        :return: mobile incidents
        """
        start, end = date_range(days, start, end)
        return self._get('mobile/incident', 'list', startDateInclusive=start,
                         endDateExclusive=end)

    def get_page(self, crawl_guid, page_guid):
        """
        retrieve a page and return it

        :param crawl_guid: crawl GUID
        :param page_guid: page GUID
        :return: requested page
        """
        return self._get('page', '%s/%s' % (crawl_guid, page_guid))

    def get_page_dom(self, crawl_guid, page_guid):
        """
        retrieve a page and return its DOM

        :param crawl_guid: crawl GUID
        :param page_guid: page GUID
        :return: requested page
        """
        return self._get('page', '%s/%s/dom' % (crawl_guid, page_guid))

    def get_page_response(self, crawl_guid, page_guid):
        """
        retrieve a page and return it

        :param crawl_guid: crawl GUID
        :param page_guid: page GUID
        :return: requested page
        """
        return self._get('page', '%s/%s/response' % (crawl_guid, page_guid))

    def get_page_child_dom(self, crawl_guid, page_guid, child_guid):
        """
        retrieve a page and return its DOM

        :param crawl_guid: crawl GUID
        :param page_guid: page GUID
        :param child_guid: child GUID
        :return: requested page
        """
        return self._get('page', '%s/%s/%s/dom' % (crawl_guid, page_guid,
                                                   child_guid))

    def get_page_child_dom_text(self, crawl_guid, page_guid, child_guid):
        """
        retrieve a page and return its DOM text

        :param crawl_guid: crawl GUID
        :param page_guid: page GUID
        :param child_guid: child GUID
        :return: requested page
        """
        return self._get('page', '%s/%s/%s/domText' % (crawl_guid, page_guid,
                                                       child_guid))

    def get_page_child_response(self, crawl_guid, page_guid, child_guid):
        """
        retrieve a page and return its response

        :param crawl_guid: crawl GUID
        :param page_guid: page GUID
        :param child_guid: child GUID
        :return: requested page
        """
        return self._get('page', '%s/%s/%s/response' % (crawl_guid, page_guid,
                                                        child_guid))

    def get_project_list(self):
        """
        List all projects.
        """
        return self._get('project', 'list')

    def get_project_keywords(self, project_id):
        """
        List all keywords associated to specified project.

        :param project_id: Integer ID of the project
        :return: Keywords of project
        """
        return self._get('project', '%d/keywords' % project_id)

    def get_proxy_ip(self, ip):
        """
        Lookup a proxy by IP

        :param ip: ip address of proxy
        :return: proxy
        """
        return self._get('proxy', 'ip/%s' % ip)

    def get_zlist_urls(self, days=1, start=None, end=None):
        """
        Get the current zlist urls.

        :param days: How many days you want to grab
        :param start: Which date to start from, use time_format.
        :param end: Date to end, use time_format.
        :return: all URLs
        """
        start, end = date_range(days, start, end)
        return self._get('zlist', 'urls', start=start, end=end)

    def post_whois(self, domain=None, email=None, name_server=None, raw=None,
                   max_results=100):
        """
        Query whois results for a domain, email, name_server.
        Allows * for wildcard.

        :param domain: Domain to query
        :param email: email address to query
        :param name_server: name server to query
        :param raw: raw data to query
        :param max_results: max results to return, default 100
        :return: list of domain dictionaries
        """
        data = {'maxResults': max_results}
        set_if(data, 'domain', domain)
        set_if(data, 'email', email)
        set_if(data, 'nameServer', name_server)
        set_if(data, 'raw', raw)
        return self._post('whois', 'query', data)

    def get_inventory(self, asset_id=None):
        """
        Retrieve a single inventory item by its assetID

        :param asset_id: assetID for the item to retreive
        :return inventory item data
        """
        # actual_url = 'inventory/' + asset_id
        return self._get('inventory', asset_id, '')

    def post_inventory_search(self, query_string, filter):
        """
        Search the inventory based on a query and filters

        :param query
        :param filters
        :return inventory search results
        """

        nested_filters_list = [filter]
        filter_dict = {'filters': nested_filters_list}
        filters_list = [filter_dict]
        data = {'query': query_string, 'filters': filters_list}

        return self._post('inventory', 'search', data)

    def get_v2_events(self, days=1, start=None, end=None, count=50, offset=0):
        """
        Get inventory items for workspace.  Can behave in one of 3 ways:

        1. Provided filter. If a filter (dict) is provided, this dict will be
           sent to the Inventory endpoint unaltered.

        2. Date range. If start and/or end dates are provided, a filter will be
        constructed for Inventory items created or updated within that period.
        3. Asset types. A list of asset types can be provided which will be
           used to query assets of that type. Available asset types are:
           ['ALL', 'WEB_SITE', 'NAME_SERVER', 'MAIL_SERVER', 'HOST', 'DOMAIN',
            'IP_BLOCK', 'ASN', 'SSL_CERT', 'CONTACT']

        :param domain: Domain to query
        :param email: email address to query
        :param name_server: name server to query
        :param max_results: max results to return, default 100
        :return: list of domain dictionaries
        """
        start, end = date_range(days, start, end)
        event_filter = {
            "query": "optional",
            "filters": [
                {
                    "filters": [
                        {
                            "field": "createdAt",
                            "value": start,
                            "type": "GTE",
                        }
                    ]
                }
            ]
        }
        return self._post('event', 'search', event_filter, count=count,
                          offset=offset)

    def search_inventory(self, filter=None, start=None, end=None,
                         asset_types=None, offset=None, count=None,
                         scroll=None):
        if asset_types is None:
            asset_types = ['ALL']
        flt = {}
        if filter:
            flt = filter
        elif start or end:
            start = start.strftime(TIME_FORMAT_ISO)
            end = end.strftime(TIME_FORMAT_ISO)

            flt = {"filters": [{
                "filters": [
                    {
                        "field": "firstSeen",
                        "value": start + "," + end,
                        "type": FilterOperation.Between,
                    },
                    {
                        "field": "lastChanged",
                        "value": start + "," + end,
                        "type": FilterOperation.Between,
                    }
                ]
            }]}

        if 'ALL' not in asset_types:
            flt['filters'].append({
                "filters": [{
                    "field": "assetType",
                    "type": FilterOperation.In,
                    "value": ",".join(asset_types),
                }]
            })

        if scroll is not None:
            return self._post('inventory', 'search', flt, scroll=scroll)
        else:
            return self._post('inventory', 'search', flt, count=count,
                              offset=offset)



#!/usr/bin/env python

import sys
import json

from riskiq.api import Client
from riskiq.render import renderer

def main():
    import argparse
    parser = argparse.ArgumentParser()

    parser.add_argument('-j', '--json', action="store_true", dest='as_json',
        help="Output as JSON")
    parser.add_argument('--days', '-d', default=1, type=int,
        help='days to query')
    parser.add_argument('--start', '-s', default=None,
        help='start datetime in "yyyy-mm-dd HH:MM:SS" format '
            '(or "today HH:MM:SS")')
    parser.add_argument('--end', '-e', default=None,
        help='end datetime in "yyyy-mm-dd HH:MM:SS" format '
            '(or "today HH:MM:SS")')
    args = parser.parse_args()

    client = Client.from_config()

    kwargs = {'as_json': args.as_json}
    kwargs['days'] = args.days
    kwargs['start'] = args.start
    kwargs['end'] = args.end
    
    data = client.get_zlist_urls(days=args.days, start=args.start,
        end=args.end)

    if args.as_json:
        print(json.dumps(data, indent=4))
    elif data:
        print(renderer(data, 'zlist/urls'))

if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        pass



#!/usr/bin/env python
''' riskiq.cli.tests
Unit tests using nose
'''
import sys
import os
import json
from StringIO import StringIO
from mock import MagicMock
from nose import with_setup

ROOT_DIR = os.path.dirname(__file__)

def load_data(filename):
    path = os.path.join(ROOT_DIR, 'testdata', filename)
    with open(path) as f:
        return json.load(f)

RETURN_VALUES = {
    'blacklist_incident': load_data('zief.pl.json'),
}

GLOBALS = {}

def ret_val(x):
    return RETURN_VALUES[x]

def setup_blacklist():
    client = MagicMock()
    client.get_blacklist_incident.return_value = ret_val('blacklist_incident')
    args = MagicMock()
    args.urls = ['zief.pl']
    stdout = sys.stdout
    buf = StringIO()
    sys.stdout = buf
    GLOBALS['client'] = client
    GLOBALS['args'] = args
    GLOBALS['stdout'] = stdout
    GLOBALS['buf'] = buf

def teardown_blacklist():
    sys.stdout = GLOBALS['stdout']

@with_setup(setup_blacklist, teardown_blacklist)
def test_bl_incident_oneline():
    from riskiq.cli.blacklist.incident import run
    client, args = GLOBALS['client'], GLOBALS['args']
    kwargs = {'oneline': True}
    run(client, args, kwargs)
    out = GLOBALS['buf'].read()



#!/usr/bin/env python
''' riskiq.cli.blacklist.malware
'''
from riskiq.cli.util import templated, six_hours
from riskiq.cli.parser import add_timerange_args, add_render_args
from riskiq.cli.blacklist import FILTERS, CONFIDENCES

def add_parser(subs):
    parser = subs.add_parser('malware',
        help='query RiskIQ suspicious binary feed for all samples stored within a '
            'given period')
    parser.add_argument('--filter', '-f', default=None,
        help='filter to one of "blackhole", "sakura" or "exploitKit"')
    parser.add_argument('--confidence', '-c', choices=['H', 'M', 'L'],
        default=None, help='restrict results to malicious probability of '
        '[H]igh, [M]edium, or [L]ow')
    add_timerange_args(parser)
    add_render_args(parser)

@templated('blacklist/malware')
def run(client, args, kwargs):
    if kwargs.get('six_hours'):
        kwargs['start'] = six_hours()
        del kwargs['six_hours']
    if kwargs['filter'] not in (None, ) + FILTERS:
        raise ValueError('Invalid filter.\nMust be one of %s' % str(FILTERS))
    if kwargs['confidence'] not in (None, ) + CONFIDENCES:
        raise ValueError('Invalid confidence.\n'
            'Must be one of %s' % str(CONFIDENCES))
    blacklist_filter = kwargs['filter']
    del kwargs['filter']
    data = client.get_blacklist_malware(blacklist_filter=blacklist_filter,
        **kwargs)
    return data, kwargs



#!/usr/bin/env python
''' riskiq.cli.parser
Generic argparse added arguments
'''

def add_timerange_args(parser):
    parser.add_argument('--days', '-d', default=1, type=int,
        help='number of days to query')
    parser.add_argument('--six-hours', '-6', action='store_true',
        help='request last 6 hours of data')
    parser.add_argument('--start', '-s', default=None,
        help='start datetime in YYYY-MM-DD HH:MM:SS format')
    parser.add_argument('--end', '-e', default=None,
        help='end datetime in YYYY-MM-DD HH:MM:SS format')

def add_render_args(parser, verbose=False):
    parser.add_argument('-l', '--oneline', action="store_true",
        help="output one entry per line")
    parser.add_argument('-j', '--json', action="store_true",
        dest='as_json', help="output raw JSON response")
    parser.add_argument('-T', '--template',
        help='path to custom template')
    if verbose:
        parser.add_argument('-v', '--verbose', action="store_true",
            help="output additional incident data in default templates")



__author__ = 'RiskIQ Research'
__version__ = '0.1-ALPHA'

import json


class GenericOutput(object):
    def __init__(self, results):
        api_results = []
        if type(results) is list:
            for api_result in results:
                if 'records' in api_result:
                    r = api_result.pop('records')
                    api_results.extend(r)
                else:
                    api_results.append(api_result)
        elif type(results) is dict:
            if 'records' in results:
                api_results = results['records']
            else:
                api_results = [api_results]
        self._results = api_results

    def _format_header(self, header_content):
        char_len = 80
        return '-- %s ' % header_content + '-' * (char_len-len(header_content))

    @property
    def results(self):
        return self._results

    @property
    def csv(self):
        raise NotImplemented('CSV Output Not Implemented')

    @property
    def json(self):
        return json.dumps(self._results, indent=4, separators=(',', ': '))

    @property
    def count(self):
        return list(self._results)

    @property
    def text(self):
        return str(self._results)

    # XXX Duplicate method name?
    @property
    def count(self):
        return len(self._results)


class PassiveDNS(GenericOutput):
    @property
    def text(self):
        r = list()
        for record in self._results:
            r.append(self._format_header(record['name']))
            r.append('Times Seen: %s' % record['count'])
            r.append('Record Type: %s' % record['rrtype'])
            r.append('Record First Seen: %s' % record['firstSeen'])
            r.append('Record Last Seen: %s' % record['lastSeen'])
            r.append('Record Responses: %s' % '\n                  '.join(record['data']))
        return '\n'.join(r)

    @property
    def csv(self):
        return


class BlacklistIncident(GenericOutput):

    @property
    def text(self):
        return repr(self._results)


class BlacklistEntry(GenericOutput):

    @property
    def text(self):
        r = list()
        for record in self._results:
            r.append(self._format_header(record['hostname']))
            r.append('Match URL: %s' % record['url'])
            r.append('Alexa Rank: %s' % record['rank'])
            r.append('Match Type: %s' % record['matchType'])
            r.append('Description: %s' % record['description'])
            r.append('Score: %s' % record['score'])
            if 'entries' in record:
                e = []
                entries = record['entries']
                for entry in entries:
                    if 'type' in entry:
                        e.append('-- %s' % entry.pop('type'))
                        for k, v in entry.items():
                            e.append('%s: %s' % (k, v))
                entries = "\n         ".join(e)
                r.append('Entries: %s' % entries)
            attributes = [i.capitalize() for i in ['malware', 'phishing', 'spam'] if record[i]]
            r.append('Flags: %s' % ", ".join(attributes))
        return "\n".join(r)

    @property
    def short(self):
        """Report blacklisted hosts by simply printing hostname"""
        r = list()
        for record in self._results:
            r.append(record['hostname'])
        return "\n".join(r)

    @property
    def oneline(self):
        """Report blacklisted hosts with listing data on a single line"""
        r = list()
        for record in self._results:
            entry_set = []
            # Set a descriptive filler word when the description for an entry is
            # empty in order to avoid a confusing empty void in the output
            record['description'] = record['description'] if record['description'] else 'NODESC'
            for entry in record['entries']:
                entry_text = entry['type']
                if 'id' in entry:
                    entry_text += "/" + str(entry['id'])
                if 'detectedAt' in entry:
                    entry_text += "/" + entry['detectedAt']
                if 'description' in entry and record['description'] == '':
                    entry_text += "/" + entry['description']
                entry_set.append(entry_text)
            record_line = "{record[url]} | {record[description]} | {entry_blob}".format(record=record, entry_blob=', '.join(entry_set))
            r.append(record_line)
        return "\n".join(r)




#!/usr/bin/env python
''' riskiq.cli.blacklist.lookup
'''
from riskiq.cli.util import templated, stdin
from riskiq.cli.parser import add_timerange_args, add_render_args

def add_parser(subs):
    parser = subs.add_parser('lookup', help='look up URL/host/domain on '
        'RiskIQ Global Blacklist (GBL)')
    parser.add_argument('urls', nargs='+', metavar='URL', help='URL/host/domain for which to query')
    add_render_args(parser)

@templated('blacklist/lookup', yielding=True)
def run(client, args, kwargs):
    urls = stdin(args.urls)
    for url in urls:
        data = client.get_blacklist_lookup(url)
        if not data:
            continue
        yield {url: data}, kwargs



#!/usr/bin/env python
''' riskiq.cli.blacklist.incidentlist
'''
from riskiq.cli.util import templated, six_hours
from riskiq.cli.parser import add_timerange_args, add_render_args

def add_parser(subs):
    parser = subs.add_parser('incidentlist',
        help='query blacklist incidents within given timeframe')
    parser.add_argument('--all-workspace-crawls', '-a',
        action='store_true', help='filter crawls to those on workspace')
    parser.add_argument('--timeout', '-t', type=float,
        default=None, help='socket timeout in seconds')
    add_timerange_args(parser)
    add_render_args(parser, verbose=True)

@templated('blacklist/incidentlist')
def run(client, args, kwargs):
    if kwargs.get('six_hours'):
        kwargs['start'] = six_hours()
        del kwargs['six_hours']
    data = client.get_blacklist_incident_list(**kwargs)
    return data, kwargs



#!/usr/bin/env python
from riskiq.config import Config
from riskiq.output import GenericOutput
from argparse import ArgumentParser
import sys

def show_config(config):
    print("\nCurrent Configuration:\n")
    for k, v in sorted(config.config.items()):
        print("{0:15}: {1}".format(k, v))

def main():
    parser = ArgumentParser()
    subs = parser.add_subparsers(dest='cmd')

    setup_parser = subs.add_parser('setup')
    setup_parser.add_argument('token', help='API token')
    setup_parser.add_argument('private_key', help='API private key')
    setup_parser.add_argument('--http-proxy', '--http', default='',
        help='proxy to use for http requests')
    setup_parser.add_argument('--https-proxy', '--https', default='',
        help='proxy to use for https requests')

    show_parser = subs.add_parser('show',
        help='show current API configuration')
    args = parser.parse_args()

    if args.cmd == 'show':
        config = Config()
        show_config(config)
    elif args.cmd == 'setup':
        config_options = {}
        config_options['api_token'] = args.token
        config_options['api_private_key'] = args.private_key
        config_options['http_proxy'] = args.http_proxy
        config_options['https_proxy'] = args.https_proxy
        config = Config(**config_options)
        show_config(config)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
''' riskiq.cli.blacklist.incident
'''
from riskiq.cli.util import templated, stdin
from riskiq.cli.parser import add_timerange_args, add_render_args

CHUNK_SIZE = 1000

def add_parser(subs):
    parser = subs.add_parser('incident', help='query blacklist incident data '
        'by given URL/host/domain')
    parser.add_argument('urls', nargs='+', metavar='URL',
        help='URL/host/domain for which to query')
    parser.add_argument('--start-index', '--si', type=int,
        help='start index, for pagination (default retrieves all data)')
    parser.add_argument('--max-results', '--mr', type=int,
        help='max results to return (default 10 if --start-index given)')
    add_render_args(parser, verbose=True)

@templated('blacklist/incident', yielding=True)
def run(client, args, kwargs):
    urls = stdin(args.urls)
    if args.start_index is None and args.max_results is None:
        for url in urls:
            chunk_i, data_ct = 0, 0
            data = {}
            incidents = []
            while True:
                data_i = client.get_blacklist_incident(url, start_index=chunk_i,
                    max_results=CHUNK_SIZE)
                data.update(data_i)
                incidents += data_i['incident']
                size_data = len(data_i['incident'])
                data_ct += size_data
                chunk_i += size_data
                if data_ct >= data_i['totalResults']:
                    break
            if not incidents:
                continue
            data['incident'] = incidents
            yield {url: data}, kwargs
    else:
        blkwargs = {}
        for url_param in ('start_index', 'max_results'):
            if getattr(args, url_param) is not None:
                blkwargs[url_param] = getattr(args, url_param)
        for url in urls:
            data = client.get_blacklist_incident(url, **blkwargs)
            yield {url: data}, kwargs
